package org.bioconductor.packages.rservices;

import java.util.ArrayList;

public class RFileReferences extends RObject {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected FileReferences fileReferences = null;

	/**
	 * 
	 */
	public RFileReferences() {
		fileReferences = new FileReferences();
	}

	/**
	 * @param fileReferences
	 */
	public RFileReferences(FileReferences fileReferences) {
		this.fileReferences = fileReferences;
	}

	/**
	 * @param urls
	 * @param localNames
	 * @param types
	 * @throws Exception
	 */
	public RFileReferences(RChar urls, RChar localNames, RChar types)
			throws Exception {
		if (urls.length() != localNames.length()
				|| urls.length() != types.length()) {
			throw new Exception(
					"'urls', 'localNames' and 'types' must have same length()");
		}
		fileReferences = new FileReferences(urls.length());
		for (int i = 0; i < urls.length(); ++i) {
			FileReference ref = new FileReference(urls.getValue()[i],
					localNames.getValue()[i], types.getValue()[i]);
			fileReferences.getFileReferences().add(ref);
		}
	}

	/**
	 * @return The size (number of elements) of the file references
	 */
	public int size() {
		return fileReferences.getFileReferences().size();
	}

	/**
	 * @return The URLs of the file refererences
	 */
	public RChar getUrls() {
		ArrayList<FileReference> refs = fileReferences.getFileReferences();
		String[] urls = new String[refs.size()];
		for (int i = 0; i < refs.size(); ++i)
			urls[i] = refs.get(i).getUrl();
		return new RChar(urls);
	}

	/**
	 * @param urls
	 * @throws Exception
	 */
	public void setUrls(RChar urls) throws Exception {
		if (size() != urls.length())
			throw new Exception(
					"'urls' must be same length as 'fileReferences'");
		ArrayList<FileReference> refs = fileReferences.getFileReferences();
		for (int i = 0; i < urls.length(); ++i) {
			refs.get(i).setUrl(urls.getValue()[i]);
		}
	}

	/**
	 * @return The local names (file names, without directory path; perhaps
	 *         different from the file name component of the corresponding url
	 */
	public RChar getLocalNames() {
		ArrayList<FileReference> refs = fileReferences.getFileReferences();
		String[] localNames = new String[refs.size()];
		for (int i = 0; i < refs.size(); ++i)
			localNames[i] = refs.get(i).getLocalName();
		return new RChar(localNames);
	}

	/**
	 * @param localNames
	 * @throws Exception
	 */
	public void setLocalNames(RChar localNames) throws Exception {
		if (size() != localNames.length())
			throw new Exception(
					"'localNames' must be same length as 'fileReferences'");
		ArrayList<FileReference> refs = fileReferences.getFileReferences();
		for (int i = 0; i < localNames.length(); ++i) {
			refs.get(i).setUrl(localNames.getValue()[i]);
		}
	}

	/**
	 * @return The file types (e.g., 'CEL', "XLS', 'TXT", perhaps different from
	 *         the name implied by the extention part of the localName
	 */
	public RChar getTypes() {
		ArrayList<FileReference> refs = fileReferences.getFileReferences();
		String[] types = new String[refs.size()];
		for (int i = 0; i < refs.size(); ++i)
			types[i] = refs.get(i).getType();
		return new RChar(types);
	}

	/**
	 * @param types
	 * @throws Exception
	 */
	public void setTypes(RChar types) throws Exception {
		if (size() != types.length())
			throw new Exception(
					"'types' must be same length as 'fileReferences'");
		ArrayList<FileReference> refs = fileReferences.getFileReferences();
		for (int i = 0; i < types.length(); ++i) {
			refs.get(i).setType(types.getValue()[i]);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result
				+ ((fileReferences == null) ? 0 : fileReferences.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final RFileReferences other = (RFileReferences) obj;
		if (fileReferences == null) {
			if (other.fileReferences != null)
				return false;
		} else if (!fileReferences.equals(other.fileReferences))
			return false;
		return true;
	}

}
