.env <- new.env()

.setUp <- function() {
    assign("wd", getwd(), envir=.env)
    setwd(tempdir())
}

.tearDown <- function() {
    setwd(.env$wd)
}

labels <- c("Raw", "Char", "Logical", "Integer", "Numeric", "Complex")
types <- c("raw", "character", "logical", "integer", "numeric", "complex")
names(types) <- labels
ClassLabels <- c("Matrix", "Array")

testSimpleCreation <- function() {

    m <- matrix(1,nrow=2,ncol=2)
    a <- array(1, dim=c(2,2,2))
    for (lbl in labels) {
        for (cl in ClassLabels) checkTrue(validObject(new(paste(lbl, cl, sep=""))))
        mode(m) <- types[[lbl]]
        mode(a) <- types[[lbl]]
        checkTrue(validObject(obj <- new(paste(lbl, "Array", sep=""), a)))
        checkTrue(all.equal(dim(obj), c(2,2,2)))
        checkTrue(validObject(obj <- new(paste(lbl, "Matrix", sep=""), m)))
        checkTrue(all.equal(dim(obj), c(2,2)))
    }
}

testTypeInfoValidIO <- function() {
    ## really testing typeInfo functionality
    f <- function(x, toType, toClass) {
        x <- unclass(x)
        mode(x) <- toType
        x <- new(toClass, x)
        return(x)
    }

    for (lbl in labels) {
        a <- array(1, dim=c(2,2,2))
        m <- matrix(1, nrow=2,ncol=2)
        mode(m) <- types[[lbl]]
        mode(a) <- types[[lbl]]
        for (toLbl in labels) {
            ## Matrix
            fromClass <- paste(lbl, "Matrix", sep="")
            toClass <- paste(toLbl, "Matrix", sep="")
            ff <- f
            typeInfo(ff) <-
              SimultaneousTypeSpecification(TypedSignature(x=fromClass),
                                            returnType=toClass)
            obj <- ff(new(fromClass, m), types[[toLbl]], toClass)
            checkTrue(class(obj)==toClass)
            checkTrue(validObject(obj))
            ## Array
            fromClass <- paste(lbl, "Array", sep="")
            toClass <- paste(toLbl, "Array", sep="")
            ff <- f
            typeInfo(ff) <-
              SimultaneousTypeSpecification(TypedSignature(x=fromClass),
                                            returnType=toClass)
            obj <- ff(new(fromClass, a), types[[toLbl]], toClass)
            checkTrue(class(obj)==toClass)
            checkTrue(validObject(obj))
        }
    }
}

testTypeInfoInvalidInput <- function() {
    ## really testing typeInfo functionality
    f <- function(x, toType, toClass) {
        x <- unclass(x)
        mode(x) <- toType
        x <- new(toClass, x)
        return(x)
    }

    for (i in seq(along=labels)) {
        lbl <- labels[[i]]
        badLbl <- labels[[1 + (i %% length(labels))]]
        a <- array(1, dim=c(2,2,2))
        m <- matrix(1, nrow=2,ncol=2)

        mode(m) <- types[[badLbl]]
        mode(a) <- types[[badLbl]]

        badClass <- paste(badLbl, "Matrix", sep="")
        fromClass <- paste(lbl, "Matrix", sep="")
        toClass <- "NumericMatrix"
        ff <- f
        typeInfo(ff) <-
          SimultaneousTypeSpecification(TypedSignature(x=fromClass),
                                        returnType=toClass)
        
        checkException(obj <- ff(new(badClass, m), "numeric", toClass), silent=TRUE)
    }
}

testGenerateDataMap <- function() {
    typeMode <- new("javalib")
    deployMode <- new("jms")
    lookup <- RWebServices:::newLookup(typeMode)
    incr <- 0
    for (lbl in labels)
      for (cl in ClassLabels) {
          incr <- incr + 1
          klass <- paste(lbl, cl, sep="")
          res <- generateDataMap(klass,
                                 javaToR=TRUE,
                                 deployMode=deployMode,
                                 typeMode=typeMode,
                                 lookup=RWebServices:::newLookup(typeMode))
          checkTrue(exists(klass, where=res, inherits=FALSE))
      }
    checkTrue(incr==prod(length(labels), length(ClassLabels)))
}

testMixedClass <- function() {
    typeMode <- new("javalib")
    deployMode <- new("jms")
    oldw <- options(warn=2)
    res <- generateDataMap("MixedNumericMatrixClass",
                           javaToR=TRUE,
                           deployMode=deployMode,
                           typeMode=typeMode,
                           lookup=RWebServices:::newLookup(typeMode))
    options(oldw)
    checkTrue(exists("MixedNumericMatrixClass", envir=res))
}
