package org.bioconductor.packages.rservices;

/*
 These test the converters between R and Java.
 Specifically, methods below pass simple data
 from Java to R and get the results back. This
 round-trip should test many of the converters.
*/

import java.util.*;
import org.omegahat.R.Java.ROmegahatInterpreter;
import org.omegahat.R.Java.REvaluator;
import org.bioconductor.packages.rservices.*;
import junit.framework.JUnit4TestAdapter;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.Ignore;
import static org.junit.Assert.*;

public class RJTypeTest extends org.bioconductor.packages.rservices.TypeTestUtil {

    /**                                                                                          
     * Used for backward compatibility (IDEs, Ant and JUnit 3.x text runner)
     */
    public static junit.framework.Test suite() {
        return new JUnit4TestAdapter(RJTypeTest.class);
    }

    /**
     * Sets up the test fixture; Called before all the tests; Called only once.
     */
    @BeforeClass
    public static void loadRWebServicesConverter() throws Exception {
        e.voidEval("regAddonCvt2()");
    }
    
    @Test
    public void testNull() throws Exception {
      assertNull(reflectObj(null));
    }

    @Test
    public void testRaw() throws Exception {
      byte[] inputVal=new byte[0];
      assertTrue(Arrays.equals(inputVal, (byte[])reflectObj(inputVal)));
      
      inputVal = new byte[]{15};
      assertTrue(Arrays.equals(inputVal, (byte[])reflectObj(inputVal)));
      
      inputVal = new byte[]{15, 16, 17};
      assertTrue(Arrays.equals(inputVal, (byte[])reflectObj(inputVal)));
    }
    
    @Test
    public void testLogical() throws Exception {
      boolean[] inputVal=new boolean[0];
      assertTrue(Arrays.equals(inputVal, (boolean[])reflectObj(inputVal)));
      
      inputVal = new boolean[]{true};
      assertTrue(Arrays.equals(inputVal, (boolean[])reflectObj(inputVal)));
      
      inputVal = new boolean[]{true , false, true};
      assertTrue(Arrays.equals(inputVal, (boolean[])reflectObj(inputVal)));
    }
    
    @Test
    public void testNumeric() throws Exception {
      double[] inputVal=new double[0];
      assertTrue(Arrays.equals(inputVal, (double[])reflectObj(inputVal)));
      
      inputVal = new double[]{Double.NaN};
      assertTrue(Arrays.equals(inputVal, (double[])reflectObj(inputVal)));
      
      inputVal = new double[]{0.1 , Double.NaN, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY, 2};
      assertTrue(Arrays.equals(inputVal, (double[])reflectObj(inputVal)));
    }
    
    @Test
    public void testInteger() throws Exception {
      int[] inputVal=new int[0];
      assertTrue(Arrays.equals(inputVal, (int[])reflectObj(inputVal)));
      
      inputVal = new int[]{1};
      assertTrue(Arrays.equals(inputVal, (int[])reflectObj(inputVal)));
      
      inputVal = new int[]{1, 0, 100, 2};
      assertTrue(Arrays.equals(inputVal, (int[])reflectObj(inputVal)));
    }
    
    @Test
    public void testCharacter() throws Exception {
      String[] inputVal=new String[0];
      assertTrue(Arrays.equals(inputVal, (String[])reflectObj(inputVal)));
      
      inputVal = new String[]{"a"};
      assertTrue(Arrays.equals(inputVal, (String[])reflectObj(inputVal)));
      
      inputVal = new String[]{"a", "weird", "b"};
      assertTrue(Arrays.equals(inputVal, (String[])reflectObj(inputVal)));
    }
    
    @Test
    public void testComplex() throws Exception {
      RJComplex inputVal=new RJComplex();
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setReal(new double[]{1, Double.NaN, 3});
      inputVal.setImaginary(new double[]{-1, 0, 1});
      assertEquals(inputVal, reflectObj(inputVal));
    }
   
    @Test
    public void testList() throws Exception {
      Object[] inputVal=new Object[0];
      assertTrue(Arrays.deepEquals(inputVal, (Object[])reflectObj(inputVal)));

      String[] eleVal1 = new String[]{"a", "weird", "b"};
      int[] eleVal2 = new int[]{1, 0, 100, 2};
      inputVal = new Object[]{eleVal1, eleVal2,
			new Object[]{eleVal1, eleVal2}};
      assertTrue(Arrays.deepEquals(inputVal, (Object[])reflectObj(inputVal)));
    }

    @Test
    public void testFactor() throws Exception {
	RJFactor inputVal=new RJFactor();
      	assertEquals(inputVal, reflectObj(inputVal));
	
	String[] flevels=new String[]{"1", "2", "3", "4", "5"};
	int[] fcode=new int[]{4, 3, 2, 0, 1, -1, 3};
      	inputVal.setLevels(flevels);
      	inputVal.setCode(fcode);
      	assertEquals(inputVal, reflectObj(inputVal));
    }

    @Test
    public void testDataFrame() throws Exception {
      RJDataFrame inputVal=new RJDataFrame();
      assertEquals(inputVal, reflectObj(inputVal));
	
      String[] eleVal1 = new String[]{"a", "weird", "b"};
      int[] eleVal2 = new int[]{1, 0, 100};
      Object[] listVal = new Object[]{eleVal1, eleVal2};
      inputVal.setData(listVal);
      inputVal.setRowNames(new String[]{"r1", "r2", "r3"});
	  inputVal.setColNames(new String[]{"c1", "c2"});
      assertEquals(inputVal, reflectObj(inputVal));
    }

	@Test
	public void testDataFrameColnames() throws Exception {
		RJDataFrame inputVal = 
			new RJDataFrame(new Object[] { 
					new int[] { 1, 2, 3 },
					new String[] { "foo", "bar", "baz" }
				},
				new String[] { "3", "2", "1" },
				new String[] { "Nums", "Lets" });
		assertEquals(inputVal, reflectObj(inputVal));
		System.out.println("done testDataFrameColnames 1");
		RJDataFrame obj = (RJDataFrame) reflectObj(inputVal);
		System.out.println("rownames:" + inputVal.getRowNames().getClass().getName());
		System.out.println("colnames:" + inputVal.getColNames().getClass().getName());
		System.out.println("data[0]:" + inputVal.getData()[0].getClass().getName());		
		System.out.println("data[1]:" + inputVal.getData()[1].getClass().getName());

		System.out.println("rownames:" + obj.getRowNames().getClass().getName());
		System.out.println("colnames:" + obj.getColNames().getClass().getName());
		System.out.println("data[0]:" + obj.getData()[0].getClass().getName());		
		System.out.println("data[1]:" + obj.getData()[1].getClass().getName());
		assertEquals(inputVal.getRowNames(), obj.getRowNames());
		System.out.println("done testDataFrameColnames 2");
		assertEquals(inputVal.getColNames(), obj.getColNames());
		System.out.println("done testDataFrameColnames 3");
		assertEquals(inputVal, obj);
		System.out.println("done testDataFrameColnames 4");
	}

    @Test
    public void testIntegerArray() throws Exception {
      int[] value=new int[]{1, 2, 3, 4, 5, 6};
      int[] dim=new int[]{2, 3, 1};
      Object[] dimnames=null;
      RJIntegerArray inputVal=new RJIntegerArray(value, dim, dimnames);
      assertEquals(inputVal, (RJIntegerArray)reflectObj(inputVal));
    }

    @Test
    public void testNumericArray() throws Exception {
      double[] value=new double[]{0.1 , Double.NaN, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY, 2, 6};
      int[] dim=new int[]{2, 3, 1};
      Object[] dimnames=null;
      RJNumericArray inputVal=new RJNumericArray(value, dim, dimnames);
      assertEquals(inputVal, (RJNumericArray)reflectObj(inputVal));
    }

    @Test
    public void testCharArray() throws Exception {
      String[] value=new String[]{"a", "b", "c", "d", "e", "f"};
      int[] dim=new int[]{2, 3, 1};
      Object[] dimnames=null;
      RJCharArray inputVal=new RJCharArray(value, dim, dimnames);
      assertEquals(inputVal, (RJCharArray)reflectObj(inputVal));
    }

    @Test
    public void testLogicalArray() throws Exception {
      boolean[] value=new boolean[]{true, false, false, true};
      int[] dim=new int[]{2, 2, 1};
      Object[] dimnames=new Object[]{new String[]{"a1", "a2"}, new String[]{"b1", "b2"}, null};
      RJLogicalArray inputVal=new RJLogicalArray(value, dim, dimnames);
      assertEquals(inputVal, (RJLogicalArray)reflectObj(inputVal));
    }

    @Test
    public void testRawArray() throws Exception {
      byte[] value=new byte[]{15, 16, 17, 18};
      int[] dim=new int[]{2, 2, 1};
      Object[] dimnames=null;
      RJRawArray inputVal=new RJRawArray(value, dim, dimnames);
      assertEquals(inputVal, (RJRawArray)reflectObj(inputVal));
    }

    @Test
    public void testComplexArray() throws Exception {
	RJComplex value=new RJComplex();
      	value.setReal(new double[]{1, Double.NaN, 3});
      	value.setImaginary(new double[]{-1, 0, 1});
      	int[] dim=new int[]{1, 3, 1};
      	Object[] dimnames=null;
      	RJComplexArray inputVal=new RJComplexArray(value, dim, dimnames);
      	assertEquals(inputVal, (RJComplexArray)reflectObj(inputVal));
    }

    @Test
    public void testIntegerMatrix() throws Exception {
      int[] value=new int[]{1, 2, 3, 4, 5, 6};
      int[] dim=new int[]{2, 3};
      Object[] dimnames=null;
      RJIntegerMatrix inputVal=new RJIntegerMatrix(value, dim, dimnames);
      assertEquals(inputVal, (RJIntegerMatrix)reflectObj(inputVal));
    }

    @Test
    public void testNumericMatrix() throws Exception {
      double[] value=new double[]{0.1 , Double.NaN, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY, 2, 6};
      int[] dim=new int[]{2, 3};
      Object[] dimnames=null;
      RJNumericMatrix inputVal=new RJNumericMatrix(value, dim, dimnames);
      assertEquals(inputVal, (RJNumericMatrix)reflectObj(inputVal));
    }

    @Test
    public void testCharMatrix() throws Exception {
      String[] value=new String[]{"a", "b", "c", "d", "e", "f"};
      int[] dim=new int[]{2, 3};
      Object[] dimnames=null;
      RJCharMatrix inputVal=new RJCharMatrix(value, dim, dimnames);
      assertEquals(inputVal, (RJCharMatrix)reflectObj(inputVal));
    }

    @Test
    public void testLogicalMatrix() throws Exception {
      boolean[] value=new boolean[]{true, false, false, true};
      int[] dim=new int[]{2, 2};
      Object[] dimnames=null;
      RJLogicalMatrix inputVal=new RJLogicalMatrix(value, dim, dimnames);
      assertEquals(inputVal, (RJLogicalMatrix)reflectObj(inputVal));
    }

    @Test
    public void testRawMatrix() throws Exception {
      byte[] value=new byte[]{15, 16, 17, 18};
      int[] dim=new int[]{2, 2};
      Object[] dimnames=null;
      RJRawMatrix inputVal=new RJRawMatrix(value, dim, dimnames);
      assertEquals(inputVal, (RJRawMatrix)reflectObj(inputVal));
    }

    @Test
    public void testComplexMatrix() throws Exception {
	RJComplex value=new RJComplex();
      	value.setReal(new double[]{1, Double.NaN, 3});
      	value.setImaginary(new double[]{-1, 0, 1});
      	int[] dim=new int[]{1, 3};
      	Object[] dimnames=null;
      	RJComplexMatrix inputVal=new RJComplexMatrix(value, dim, dimnames);
      	assertEquals(inputVal, (RJComplexMatrix)reflectObj(inputVal));
    }

    @Test
    public void testEnvironment() throws Exception {
	HashMap<String, Object>  inputVal=new HashMap<String, Object>();
      	assertTrue(RJUtil.deepEquals(inputVal, (HashMap)reflectObj(inputVal)));

	RJFactor factorVal=new RJFactor();
	String[] flevels=new String[]{"1", "2", "3", "4", "5"};
	int[] fcode=new int[]{4, 3, 2, 0, 1, -1, 3};
      	factorVal.setLevels(flevels);
      	factorVal.setCode(fcode);
      	int[] integerVal = new int[]{1, 0, 100};
	Object[] listVal = new Object[]{
				new String[]{"a", "weird", "b"},
				new int[]{1, 0}
			   };
	String[] stringVal = new String[]{"test"};
	HashMap<String, Object> envVal = new HashMap<String, Object>();
	envVal.put("my_list", listVal);
	inputVal.put("my_factor", factorVal);
      	inputVal.put("my_null", null);
      	inputVal.put("my_int", integerVal);
	inputVal.put("my_list", listVal);
	inputVal.put("my_string", stringVal);
	inputVal.put("my_env", envVal);
      	assertTrue(RJUtil.deepEquals(inputVal, (HashMap)reflectObj(inputVal)));
    }

}

